/*!
 * vue-use-parallax 1.0.1-alpha.1+50a5174
 * Copyright 2020-preset tomieric<https://github.com/tomieric> All Rights Reserved
 * Licensed under MIT
 */

'use strict';

var vue = require('vue');

/**
 *
 * vue hooks for parallax
 * @param {HTMLElement} el An area container that the mouse moves
 * @param {(Array<[] | HTMLElement>)} Scrolling elements
 */
const useParallax = (el, children = []) => {
    const queue = [...children];
    const parallx = (elem, x, y, offset = 10) => {
        elem && (elem.style.transform = `translate(${x * offset / 0.5}px, ${y * offset / 0.5}px)`);
    };
    const publish = (offsetX, offsetY) => {
        queue.forEach((children) => {
            if (Array.isArray(children)) {
                parallx(children[0], offsetX, offsetY, children[1]);
            }
            else {
                parallx(children, offsetX, offsetY);
            }
        });
    };
    const setParallax = (children = []) => {
        queue.push(...children);
    };
    const scrollParallax = (e) => {
        const offsetX = e.clientX / el.clientWidth;
        const offsetY = e.clientY / el.clientHeight;
        requestAnimationFrame(() => publish(offsetX, offsetY));
    };
    // remove eventListener
    const resetParallax = () => el && el.removeEventListener('mousemove', scrollParallax, false);
    vue.onUnmounted(resetParallax);
    // mousemove
    const startParallax = () => {
        resetParallax();
        el && el.addEventListener('mousemove', scrollParallax, false);
    };
    startParallax();
    return {
        startParallax,
        setParallax,
        resetParallax
    };
};

module.exports = useParallax;
