/*!
 * vue-use-parallax 1.0.1-alpha.1+50a5174
 * Copyright 2020-preset tomieric<https://github.com/tomieric> All Rights Reserved
 * Licensed under MIT
 */

import { onUnmounted, isRef, onMounted } from 'vue';

/**
 *
 * vue hooks for parallax
 * @param {HTMLElement} el An area container that the mouse moves
 * @param {(Array<[] | HTMLElement>)} Scrolling elements
 */
const useParallax = (el, children = []) => {
    const queue = [...children];
    const parallx = (elem, x, y, offset = 10) => {
        elem && isRef(elem) && (elem.value.style.transform = `translate(${x * offset / 0.5}px, ${y * offset / 0.5}px)`);
    };
    const publish = (offsetX, offsetY) => {
        queue.forEach((children) => {
            if (Array.isArray(children)) {
                parallx(children[0], offsetX, offsetY, children[1]);
            }
            else {
                parallx(children, offsetX, offsetY);
            }
        });
    };
    const setParallax = (children = []) => {
        queue.push(...children);
    };
    const scrollParallax = (e) => {
        if (!isRef(el))
            return;
        const offsetX = e.clientX / el.value.clientWidth;
        const offsetY = e.clientY / el.value.clientHeight;
        requestAnimationFrame(() => publish(offsetX, offsetY));
    };
    // remove eventListener
    const resetParallax = () => el && isRef(el) && el.value.removeEventListener('mousemove', scrollParallax, false);
    onUnmounted(() => resetParallax());
    // mousemove
    const startParallax = () => {
        resetParallax();
        el && isRef(el) && el.value.addEventListener('mousemove', scrollParallax, false);
    };
    onMounted(() => startParallax());
    return {
        startParallax,
        setParallax,
        resetParallax
    };
};

export default useParallax;
